searchData={"items":[{"type":"module","title":"Oidcc.Plug.AuthorizationCallback","doc":"Retrieve Token for Code Flow Authorization Callback\n\nThis plug does not send a response. Instead it will load and validate all\ntoken data and leave the rest to a controller action that will be executed\nafter.","ref":"Oidcc.Plug.AuthorizationCallback.html"},{"type":"module","title":"Via `Phoenix.Router` - Oidcc.Plug.AuthorizationCallback","doc":"```elixir\ndefmodule SampleAppWeb.Router do\n  use Phoenix.Router\n\n  # ...\n\n  pipeline :oidcc_callback do\n    plug Oidcc.Plug.AuthorizationCallback,\n      provider: SampleApp.GoogleOpenIdConfigurationProvider,\n      client_id: Application.compile_env!(:sample_app, [Oidcc.Plug.Authorize, :client_id]),\n      client_secret: Application.compile_env!(:sample_app, [Oidcc.Plug.Authorize, :client_secret]),\n      redirect_uri: \"https://localhost:4000/oidcc/callback\"\n  end\n\n  forward \"/oidcc/authorize\", to: Oidcc.Plug.Authorize,\n    init_opts: [...]\n\n  scope \"/oidcc/callback\", SampleAppWeb do\n    pipe_through :oidcc_callback\n\n    get \"/\", AuthController, :handle_callback\n    post \"/\", AuthController, :handle_callback\n  end\nend\n```","ref":"Oidcc.Plug.AuthorizationCallback.html#module-via-phoenix-router"},{"type":"module","title":"Via `Controller` - Oidcc.Plug.AuthorizationCallback","doc":"```elixir\ndefmodule SampleAppWeb.AuthController do\n  # ...\n\n  plug Oidcc.Plug.AuthorizationCallback,\n    provider: SampleApp.GoogleOpenIdConfigurationProvider,\n    client_id: Application.compile_env!(:sample_app, [Oidcc.Plug.Authorize, :client_id]),\n    client_secret: Application.compile_env!(:sample_app, [Oidcc.Plug.Authorize, :client_secret]),\n    redirect_uri: \"https://localhost:4000/oidcc/callback\"\n    when action in [:handle_callback]\n\n  def handle_callback(\n    %Plug.Conn{private: %{\n      Oidcc.Plug.AuthorizationCallback => {:ok, {token, userinfo}}\n    }},\n    _params\n  ) do\n    # Handle Success\n\n    conn\n    |> put_session(\"auth_token\", token)\n    |> put_session(\"auth_userinfo\", userinfo)\n    |> redirect(to: \"/\")\n  end\n\n  def handle_callback(\n    %Plug.Conn{private: %{\n      Oidcc.Plug.AuthorizationCallback => {:error, reason}}\n    },\n    _params\n  ) do\n    # Handle Error\n\n    conn\n    |> put_status(400)\n    |> render(\"error.html\", reason: reason)\n  end\nend\n```","ref":"Oidcc.Plug.AuthorizationCallback.html#module-via-controller"},{"type":"type","title":"Oidcc.Plug.AuthorizationCallback.error/0","doc":"","ref":"Oidcc.Plug.AuthorizationCallback.html#t:error/0"},{"type":"type","title":"Oidcc.Plug.AuthorizationCallback.opts/0","doc":"Plug Configuration Options","ref":"Oidcc.Plug.AuthorizationCallback.html#t:opts/0"},{"type":"type","title":"Options - Oidcc.Plug.AuthorizationCallback.opts/0","doc":"* `provider` - name of the `Oidcc.ProviderConfiguration.Worker`\n* `client_id` - OAuth Client ID to use for the introspection\n* `client_secret` - OAuth Client Secret to use for the introspection\n* `client_context_opts` - Options for Client Context Initialization\n* `client_profile_opts` - Options for Client Context Profiles\n* `redirect_uri` - Where to redirect for callback\n* `check_useragent` - check if useragent is the same as before the\n  authorization request\n* `check_peer_ip` - check if the client IP is the same as before the\n  authorization request\n* `retrieve_userinfo` - whether to load userinfo from the provider\n* `request_opts` - request opts for http calls to provider","ref":"Oidcc.Plug.AuthorizationCallback.html#t:opts/0-options"},{"type":"module","title":"Oidcc.Plug.Authorize","doc":"Initiate Code Flow Authorization Redirect\n\n```elixir\ndefmodule SampleAppWeb.Router do\n  use Phoenix.Router\n\n  # ...\n\n  forward \"/oidcc/authorize\", to: Oidcc.Plug.Authorize,\n    init_opts: [\n      provider: SampleApp.GoogleOpenIdConfigurationProvider,\n      client_id: Application.compile_env!(:sample_app, [Oidcc.Plug.Authorize, :client_id]),\n      client_secret: Application.compile_env!(:sample_app, [Oidcc.Plug.Authorize, :client_secret]),\n      redirect_uri: \"https://localhost:4000/oidcc/callback\"\n    ]\nend\n```","ref":"Oidcc.Plug.Authorize.html"},{"type":"module","title":"Query Params - Oidcc.Plug.Authorize","doc":"* `state` - State to relay to OpenID Provider. Commonly used for target redirect\n  URL after authorization.","ref":"Oidcc.Plug.Authorize.html#module-query-params"},{"type":"type","title":"Oidcc.Plug.Authorize.opts/0","doc":"Plug Configuration Options","ref":"Oidcc.Plug.Authorize.html#t:opts/0"},{"type":"type","title":"Options - Oidcc.Plug.Authorize.opts/0","doc":"* `scopes` - scopes to request\n* `redirect_uri` - Where to redirect for callback\n* `url_extension` - Custom query parameters to add to the redirect URI\n* `provider` - name of the `Oidcc.ProviderConfiguration.Worker`\n* `client_id` - OAuth Client ID to use for the introspection\n* `client_secret` - OAuth Client Secret to use for the introspection\n* `client_context_opts` - Options for Client Context Initialization\n* `client_profile_opts` - Options for Client Context Profiles","ref":"Oidcc.Plug.Authorize.html#t:opts/0-options"},{"type":"behaviour","title":"Oidcc.Plug.Cache","doc":"Behaviour to cache introspection / userinfo requests","ref":"Oidcc.Plug.Cache.html"},{"type":"behaviour","title":"Usage - Oidcc.Plug.Cache","doc":"* Userinfo: See `t:Oidcc.Plug.LoadUserinfo.opts/0` / `cache`\n* Introspection: See `t:Oidcc.Plug.IntrospectToken.opts/0` / `cache`","ref":"Oidcc.Plug.Cache.html#module-usage"},{"type":"callback","title":"Oidcc.Plug.Cache.get/3","doc":"Check cache if userinfo / introspection is stored for `token`.","ref":"Oidcc.Plug.Cache.html#c:get/3"},{"type":"callback","title":"Oidcc.Plug.Cache.put/4","doc":"Store userinfo / introspection for `token`.","ref":"Oidcc.Plug.Cache.html#c:put/4"},{"type":"type","title":"Oidcc.Plug.Cache.t/0","doc":"","ref":"Oidcc.Plug.Cache.html#t:t/0"},{"type":"module","title":"Oidcc.Plug.ExtractAuthorization","doc":"Extract `authorization` request header\n\nThis module should be used together with `Oidcc.Plug.IntrospectToken`,\n`Oidcc.Plug.LoadUserinfo` or `Oidcc.Plug.ValidateJwtToken`.\n\n```elixir\ndefmodule SampleAppWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :sample_app\n\n  # ...\n\n  plug Oidcc.Plug.ExtractAuthorization\n\n  plug Oidcc.Plug.RequireAuthorization, [...] # Ensure Authorization Token provided\n  plug Oidcc.Plug.IntrospectToken, [...] # Check Token via Introspection\n  plug Oidcc.Plug.LoadUserinfo, [...] # Check Token via Userinfo\n  plug Oidcc.Plug.ValidateJwtToken, [...] # Check Token via JWT validation\n\n  plug SampleAppWeb.Router\nend\n```","ref":"Oidcc.Plug.ExtractAuthorization.html"},{"type":"type","title":"Oidcc.Plug.ExtractAuthorization.opts/0","doc":"Plug Configuration Options","ref":"Oidcc.Plug.ExtractAuthorization.html#t:opts/0"},{"type":"type","title":"Options - Oidcc.Plug.ExtractAuthorization.opts/0","doc":"* `send_invalid_header_response` - Customize Error Response for invalid header","ref":"Oidcc.Plug.ExtractAuthorization.html#t:opts/0-options"},{"type":"module","title":"Oidcc.Plug.IntrospectToken","doc":"Validate extracted authorization token using introspection.\n\nSee: https://datatracker.ietf.org/doc/html/rfc7662\n\nThis module should be used together with `Oidcc.Plug.ExtractAuthorization`.\n\nThis plug will send an introspection request for every request. To avoid this,\nprovide a `cache` to `t:opts/0`.\n\n```elixir\ndefmodule SampleAppWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :sample_app\n\n  # ...\n\n  plug Oidcc.Plug.ExtractAuthorization\n\n  plug Oidcc.Plug.IntrospectToken,\n    provider: SampleApp.GoogleOpenIdConfigurationProvider,\n    client_id: Application.compile_env!(:sample_app, [Oidcc.Plug.IntrospectToken, :client_id]),\n    client_secret: Application.compile_env!(:sample_app, [Oidcc.Plug.IntrospectToken, :client_secret])\n\n  plug SampleAppWeb.Router\nend\n```","ref":"Oidcc.Plug.IntrospectToken.html"},{"type":"type","title":"Oidcc.Plug.IntrospectToken.opts/0","doc":"Plug Configuration Options","ref":"Oidcc.Plug.IntrospectToken.html#t:opts/0"},{"type":"type","title":"Options - Oidcc.Plug.IntrospectToken.opts/0","doc":"* `provider` - name of the `Oidcc.ProviderConfiguration.Worker`\n* `client_id` - OAuth Client ID to use for the introspection\n* `client_secret` - OAuth Client Secret to use for the introspection\n* `token_introspection_opts` - Options to pass to the introspection\n* `send_inactive_token_response` - Customize Error Response for inactive token\n* `cache` - Cache token introspection - See `Oidcc.Plug.Cache`","ref":"Oidcc.Plug.IntrospectToken.html#t:opts/0-options"},{"type":"module","title":"Oidcc.Plug.LoadUserinfo","doc":"Validate extracted authorization token using userinfo retrieval.\n\nSee: https://openid.net/specs/openid-connect-core-1_0.html#UserInfo\n\nThis module should be used together with `Oidcc.Plug.ExtractAuthorization`.\n\nThis plug will send a userinfo request for every request. To avoid this,\nprovide a `cache` to `t:opts/0`.\n\n```elixir\ndefmodule SampleAppWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :sample_app\n\n  # ...\n\n  plug Oidcc.Plug.ExtractAuthorization\n\n  plug Oidcc.Plug.LoadUserinfo,\n    provider: SampleApp.GoogleOpenIdConfigurationProvider,\n    client_id: Application.compile_env!(:sample_app, [Oidcc.Plug.LoadUserinfo, :client_id]),\n    client_secret: Application.compile_env!(:sample_app, [Oidcc.Plug.LoadUserinfo, :client_secret])\n\n  plug SampleAppWeb.Router\nend\n```","ref":"Oidcc.Plug.LoadUserinfo.html"},{"type":"type","title":"Oidcc.Plug.LoadUserinfo.opts/0","doc":"Plug Configuration Options","ref":"Oidcc.Plug.LoadUserinfo.html#t:opts/0"},{"type":"type","title":"Options - Oidcc.Plug.LoadUserinfo.opts/0","doc":"* `provider` - name of the `Oidcc.ProviderConfiguration.Worker`\n* `client_id` - OAuth Client ID to use for the userinfo retrieval\n* `client_secret` - OAuth Client Secret to use for the userinfo retrieval\n* `userinfo_retrieve_opts` - Options to pass to userinfo loading\n* `send_inactive_token_response` - Customize Error Response for inactive token\n* `cache` - Cache userinfo response - See `Oidcc.Plug.Cache`","ref":"Oidcc.Plug.LoadUserinfo.html#t:opts/0-options"},{"type":"module","title":"Oidcc.Plug.RequireAuthorization","doc":"Ensure authorization token provided.\n\nThis module should be used together with `Oidcc.Plug.ExtractAuthorization`.\n\n```elixir\ndefmodule SampleAppWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :sample_app\n\n  # ...\n\n  plug Oidcc.Plug.ExtractAuthorization\n\n  plug Oidcc.Plug.RequireAuthorization\n\n  # Check Token with `Oidcc.Plug.IntrospectToken`, `Oidcc.Plug.LoadUserinfo` or `Oidcc.Plug.ValidateJwtToken`\n\n  plug SampleAppWeb.Router\nend\n```","ref":"Oidcc.Plug.RequireAuthorization.html"},{"type":"type","title":"Oidcc.Plug.RequireAuthorization.opts/0","doc":"Plug Configuration Options","ref":"Oidcc.Plug.RequireAuthorization.html#t:opts/0"},{"type":"type","title":"Options - Oidcc.Plug.RequireAuthorization.opts/0","doc":"* `send_missing_token_response` - Customize Error Response for missing token","ref":"Oidcc.Plug.RequireAuthorization.html#t:opts/0-options"},{"type":"module","title":"Oidcc.Plug.ValidateJwtToken","doc":"Validate extracted authorization token by validating it as a JWT token.\n\nThis module should be used together with `Oidcc.Plug.ExtractAuthorization`.\n\n```elixir\ndefmodule SampleAppWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :sample_app\n\n  # ...\n\n  plug Oidcc.Plug.ExtractAuthorization\n\n  plug Oidcc.Plug.ValidateJwtToken,\n    provider: SampleApp.GoogleOpenIdConfigurationProvider,\n    client_id: Application.compile_env!(:sample_app, [Oidcc.Plug.ValidateJwtToken, :client_id]),\n    client_secret: Application.compile_env!(:sample_app, [Oidcc.Plug.ValidateJwtToken, :client_secret])\n\n  plug SampleAppWeb.Router\nend\n```","ref":"Oidcc.Plug.ValidateJwtToken.html"},{"type":"type","title":"Oidcc.Plug.ValidateJwtToken.opts/0","doc":"Plug Configuration Options","ref":"Oidcc.Plug.ValidateJwtToken.html#t:opts/0"},{"type":"type","title":"Options - Oidcc.Plug.ValidateJwtToken.opts/0","doc":"* `provider` - name of the `Oidcc.ProviderConfiguration.Worker`\n* `client_id` - OAuth Client ID to use for the token validation\n* `client_secret` - OAuth Client Secret to use for the token validation\n* `send_inactive_token_response` - Customize Error Response for inactive token","ref":"Oidcc.Plug.ValidateJwtToken.html#t:opts/0-options"},{"type":"exception","title":"Oidcc.Plug.Authorize.Error","doc":"Redirect URI Generation Failed\n\nCheck the `reason` field for ther exact reason","ref":"Oidcc.Plug.Authorize.Error.html"},{"type":"exception","title":"Oidcc.Plug.IntrospectToken.Error","doc":"Introspection Failed\n\nCheck the `reason` field for ther exact reason","ref":"Oidcc.Plug.IntrospectToken.Error.html"},{"type":"exception","title":"Oidcc.Plug.LoadUserinfo.Error","doc":"Retrieve Userinfo Failed\n\nCheck the `reason` field for ther exact reason","ref":"Oidcc.Plug.LoadUserinfo.Error.html"},{"type":"exception","title":"Oidcc.Plug.ValidateJwtToken.Error","doc":"Validation Failed\n\nCheck the `reason` field for ther exact reason","ref":"Oidcc.Plug.ValidateJwtToken.Error.html"},{"type":"extras","title":"README","doc":"# Oidcc.Plug\n\nPlug Integration for [`oidcc`](https://hex.pm/packages/oidcc) library.\n\n[![EEF Security WG project](https://img.shields.io/badge/EEF-Security-black)](https://github.com/erlef/security-wg)\n[![Main Branch](https://github.com/erlef/oidcc_plug/actions/workflows/branch_main.yml/badge.svg?branch=main)](https://github.com/erlef/oidcc_plug/actions/workflows/branch_main.yml)\n[![Module Version](https://img.shields.io/hexpm/v/oidcc_plug.svg)](https://hex.pm/packages/oidcc_plug)\n[![Total Download](https://img.shields.io/hexpm/dt/oidcc_plug.svg)](https://hex.pm/packages/oidcc_plug)\n[![License](https://img.shields.io/hexpm/l/oidcc_plug.svg)](https://github.com/erlef/oidcc_plug/blob/main/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/erlef/oidcc_plug.svg)](https://github.com/erlef/oidcc_plug/commits/master)\n[![Coverage Status](https://coveralls.io/repos/github/erlef/oidcc_plug/badge.svg?branch=main)](https://coveralls.io/github/erlef/oidcc_plug?branch=main)\n\n \n\n \n   \n   \n   \n \n\nOpenID Certified by [Jonatan Männchen](https://github.com/maennchen) at the\n[Erlang Ecosystem Foundation](https://github.com/erlef) of multiple Relaying\nParty conformance profiles of the OpenID Connect protocol:\nFor details, check the\n[Conformance Test Suite](https://github.com/erlef/oidcc_conformance).\n\n \n\n \n   \n   \n   \n \n\nThe development of the library and the certification is funded as an\n[Erlang Ecosystem Foundation](https://erlef.org/) stipend entered by the\n[Security Working Group](https://erlef.org/wg/security).","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"The package can be installed by adding `oidcc_plug` to your list of dependencies\nin `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:oidcc_plug, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - README","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Setup - README","doc":"```elixir\ndefmodule SampleApp.Application do\n  # ...\n\n  @impl true\n  def start(_type, _args) do\n    children = [\n      # ...\n\n      {Oidcc.ProviderConfiguration.Worker, %{\n        issuer: \"https://accounts.google.com\",\n        name: SampleApp.GoogleOpenIdConfigurationProvider\n      }},\n\n      # Start the Endpoint (http/https)\n      SampleAppWeb.Endpoint\n    ]\n\n    # See https://hexdocs.pm/elixir/Supervisor.html\n    # for other strategies and supported options\n    opts = [strategy: :one_for_one, name: SampleApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\n\n  # ...\nend\n```","ref":"readme.html#setup"},{"type":"extras","title":"Authorization Flow - README","doc":"```elixir\ndefmodule SampleAppWeb.OidccController do\n  use SampleAppWeb, :controller\n\n  plug Oidcc.Plug.Authorize,\n    [\n      provider: TestWorks.OpenIdConfigurationProvider,\n      client_id: \"client_id\",\n      client_secret: \"client_secret\",\n      redirect_uri: &__MODULE__.callback_uri/0\n    ]\n    when action in [:authorize]\n\n  plug Oidcc.Plug.AuthorizationCallback,\n    [\n      provider: TestWorks.OpenIdConfigurationProvider,\n      client_id: \"client_id\",\n      client_secret: \"client_secret\",\n      redirect_uri: &__MODULE__.callback_uri/0\n    ]\n    when action in [:callback]\n\n  @doc false\n  def callback_uri, do: url(~p\"/oidcc/callback\")\n\n  def authorize(conn, _params), do: conn\n\n  def callback(%Plug.Conn{private: %{\n    Oidcc.Plug.AuthorizationCallback => {:ok, {_token, userinfo}}}\n  } = conn, params) do\n    conn\n    |> put_session(\"oidcc_claims\", userinfo)\n    |> redirect(to: \"/\")\n  end\n\n  def callback(%Plug.Conn{private: %{\n    Oidcc.Plug.AuthorizationCallback => {:error, reason}\n  }} = conn, _params) do\n    conn\n    |> put_status(400)\n    |> render(:error, reason: reason)\n  end\nend\n```","ref":"readme.html#authorization-flow"},{"type":"extras","title":"API (Check access token header) - README","doc":"```elixir\ndefmodule SampleAppWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :sample_app\n\n  # ...\n\n  plug Oidcc.Plug.ExtractAuthorization\n\n  @client_id Application.compile_env!(:sample_app, [:openid_credentials, :client_id])\n  @client_secret Application.compile_env!(:sample_app, [:openid_credentials, :client_secret])\n\n  # Ensure Authorization Token provided\n  plug Oidcc.Plug.RequireAuthorization\n\n  # Check Token via Introspection\n  plug Oidcc.Plug.IntrospectToken,\n    provider: SampleApp.GoogleOpenIdConfigurationProvider,\n    client_id: @client_id,\n    client_secret: @client_secret\n\n  # OR: Check Token via Userinfo\n  plug Oidcc.Plug.LoadUserinfo,\n    provider: SampleApp.GoogleOpenIdConfigurationProvider,\n    client_id: @client_id,\n    client_secret: @client_secret\n\n  # OR: Check Token via JWT validation\n  plug Oidcc.Plug.ValidateJwtToken,\n    provider: SampleApp.GoogleOpenIdConfigurationProvider,\n    client_id: @client_id,\n    client_secret: @client_secret\n\n  plug SampleAppWeb.Router\nend\n```","ref":"readme.html#api-check-access-token-header"}],"content_type":"text/markdown","proglang":"elixir","producer":{"name":"ex_doc","version":"0.35.1"}}